-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp - all rights reserved.
-- 
-- For more scripts and sample code, check out 
-- 	http://www.SQLSkills.com
-- For "More than Just Training," see the Industry Experts at 
-- 	http://www.SolidQualityLearning.com
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

-- Severity
-- 1-9	Informational - Error number in black with error message
-- 		@@ERROR is NOT set
-- 10	Warning - ONLY the error message is returned
-- 		@@ERROR is NOT set		
-- 11-16	Error - USER defined error...default 16
-- 		Error number in red with error message
-- 		@@ERROR is set
-- State
-- 127 Kills client connection
-- 
USE master
go
--sp_dropmessage 900001
--sp_dropmessage 900001, 'FRENCH' -- Once you add messages in additional langs you must drop them first to drop the english message.

exec sp_addmessage @msgnum = 900001, 
	@severity = 16, 
	@msgtext = 'Value %s is invalid. Procedure %s expected a value of TRUE or FALSE for parameter %s. The default is %s.' 
	, @lang =   'US_English'
	, @with_log =  'FALSE'
go
exec sp_addmessage @msgnum = 900001, 
	@severity = 16, 
	@msgtext = 'La valeur %s est inadmissible.  Le procd %s s''est attendu  une valeur de VRAI ou de FAUX pour le paramtre %s.  Le dfaut est %s.' 
		-- translation provided by www.babblefish.com
	, @lang =   'French'
	, @with_log =  'FALSE'
go
-- Quick Test
RAISERROR (900001, 1, 1, 'string', 'string', 'string', 'string') with log
SELECT @@Error
go
RAISERROR (900001, 10, 1, 'string', 'string', 'string', 'string') with log
SELECT @@Error
go
RAISERROR (900001, 16, 1, 'string', 'string', 'string', 'string') with log
SELECT @@Error
go
CREATE PROCEDURE TestProc
(
			@Param1	varchar(5) = 'FALSE'
)
as
DECLARE @ErrorMsg	int
IF @Param1 IS NULL
	SET @Param1 = 'FALSE'

IF UPPER(@Param1) NOT IN ('TRUE', 'FALSE')
    BEGIN
	RAISERROR (900001, 16, -1, @Param1, 'TestProc', '@Param1', 'FALSE')
	RETURN @@ERROR
-- OR
	SET @ErrorMsg = @@ERROR
	-- Do other stuff...
	RETURN @ErrorMsg -- because by this statement @@ERROR = 0
    END

PRINT 'Stored Procedure completed successfully'
go
exec TestProc
-- default does kick in...

exec TestProc NULL
-- default does not kick in...

exec TestProc foo
-- default does not kick in...

-- Now test it with return status...
DECLARE @RetStat	int
exec @RetStat = TestProc
select @RetStat -- no error
	---default kicks in

DECLARE @RetStat	int
exec @RetStat = TestProc NULL
select @RetStat  -- no error
	-- default DOES NOT kick in! 
	-- You need the IF at the beginning of the proc

DECLARE @RetStat	int
EXEC @RetStat = TestProc 'foo'
IF @RetStat <> 0
BEGIN
	PRINT 'Stored Procedure failed...Please try again later!'
	-- Application Logic
END	

--Now you can even define an alert (administrative)
EXEC TestProc 'foo'