-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp - all rights reserved.
-- 
-- For more scripts and sample code, check out 
-- 	http://www.SQLSkills.com
-- For "More than Just Training," see the Industry Experts at 
-- 	http://www.SolidQualityLearning.com
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/********************************************
********* REVIEW FOR QUESTIONS *********
********************************************/

-- So you've been through the complete scenario in "Restore File Backups.sql" and are ready to 
-- review the answers and proper restore sequences!

-- Once again, begin by reviewing the backup history in MSDB
SELECT Backup_Start_Date, 
	[Name],
	[Description],
	First_LSN, 
	Last_LSN, 
	Backup_Finish_Date, 
	* -- OR Just *
FROM msdb.dbo.backupset AS s
    JOIN msdb.dbo.backupmediafamily AS m
        ON s.media_set_id = m.media_set_id
WHERE database_name = 'PubsTest'
ORDER BY 1 ASC

-- Make sure that you know where and what type each backup is - good naming conventions
-- really help here. NOTE: you could have gotten the same information from the backup
-- devices as well.
-- But how many devices do you have? Are they online?

LOAD HEADERONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
-- What are the last FULL file/filegroup backups (by position) type = 4

-- Do we know what they are? Yes, excellent naming conventions...if NOT?
LOAD FILELISTONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
	WITH FILE = 1 -- This is the position number on a multifile backup
LOAD FILELISTONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
	WITH FILE = 2 -- This is the position number on a multifile backup
--...
LOAD FILELISTONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
	WITH FILE = 10 -- This is the position number on a multifile backup

-- NOTE: It is a full filegroup backup if all files are listed from LOAD FILELISTONLY.

-- Find the last differentials...
LOAD HEADERONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
-- What are the last file/filegroup DIFFERENTIALS (by position) type = 6

-- What is the log series to restore? Check the LSN
LOAD HEADERONLY FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
--What are the last log series by LSN (by position) type = 2


/***************************************
******** ANSWER FOR QUESTION #1 ********
***************************************/

-- 1) What do you need to do to completely recover the entire database in another location?

-- This is the hardest question... A complete recover to another location requires ALL File/FileGroup 
-- backups to be restored FIRST, then Differentials, then finally all logs with the minimum LSN. 

-- First, make sure you have the tail of the log:
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FINAL Transaction Log (Tail up-to-minute)', NOINIT,
	NO_TRUNCATE
go

-- Since this is not a true failure we're going to review what we have already backed up.
-- In our case the backup sequence was:
-- 1	-- PubsTest Backup, File = pubs
-- 2	-- PubsTest Backup, Transaction Log
-- 3	-- PubsTest Backup, FileGroup = TestFG
-- 4	-- PubsTest Backup, Transaction Log
-- 5	-- PubsTest Backup, FileGroup DIFFERENTIAL = TestFG
-- 6	-- PubsTest Backup, Transaction Log
-- 7	-- PubsTest Backup, File = pubs
-- 8	-- PubsTest Backup, Transaction Log
-- 9	-- PubsTest Backup, FileGroup DIFFERENTIAL = TestFG
-- 10	-- PubsTest Backup, Transaction Log
-- 11	-- PubsTest Backup, Final Transaction Log (TAIL)

-- To setup the database we need the "pubs" file (backup number 7), then the TestFG 
-- FileGroup (backup number 3), then the last differential of the TestFG filegroup 
-- (backup number 9). This will setup the framework of the database.

RESTORE DATABASE NewPubsTest
	FILE = 'pubs'
	-- OR FILEGROUP = 'Primary'
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile1.ndf',
	MOVE 'PubsTestFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile2.ndf',
	MOVE 'PubsTestFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile3.ndf',
FILE = 7, -- this is the backup number by position
NORECOVERY
go

RESTORE DATABASE NewPubsTest
	FILEGROUP = 'TestFG'
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile1.ndf',
	MOVE 'PubsTestFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile2.ndf',
	MOVE 'PubsTestFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile3.ndf',
FILE = 3, -- this is the backup number by position
NORECOVERY
go

RESTORE DATABASE NewPubsTest
	FILEGROUP = 'TestFG'
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile1.ndf',
	MOVE 'PubsTestFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile2.ndf',
	MOVE 'PubsTestFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile3.ndf',
FILE = 9, -- this is the backup number by position
NORECOVERY
go

-- The database is setup BUT the data has not been rolled forward. We need to apply the correct
-- sequence of transaction logs. To determine the minimum effective LSN we need to look at the MIN LSN
-- for each of the last FULL FILE or last DIFFERTIALs we restored is... The lowest would be backup 
-- number 7 (not backup number 3 - because 7 updated this one with the differential changes).
-- So we need to review the LSNs and find the transaction log backup that has a MIN LSN that is LESS
-- than the MIN LSN of backup number 7. 

-- In this case I have Tlog Backups 8 and 10...but you might have a scenario where the first log to apply
-- is EARLIER than this. ALWAYS review the MIN LSNs to make sure that you have the correct sequence!

RESTORE LOG NewPubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile1.ndf',
	MOVE 'PubsTestFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile2.ndf',
	MOVE 'PubsTestFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile3.ndf',
FILE = 8, -- this is the backup number by position
NORECOVERY
go

RESTORE LOG NewPubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile1.ndf',
	MOVE 'PubsTestFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile2.ndf',
	MOVE 'PubsTestFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile3.ndf',
FILE = 10, -- this is the backup number by position
NORECOVERY 
go

RESTORE LOG NewPubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH 
	MOVE 'pubs' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs.mdf',
	MOVE 'pubs_log' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubs_Log.ldf',
	MOVE 'PubsTestFile1' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile1.ndf',
	MOVE 'PubsTestFile2' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile2.ndf',
	MOVE 'PubsTestFile3' TO 'C:\Program Files\Microsoft SQL Server\MSSQL\DATA\NewPubsTestFile3.ndf',
FILE = 11, -- this is the backup number by position
RECOVERY 
go

SELECT count(*) FROM NewPubsTest.dbo.DemoTable -- You should have 22000 rows in NewPubsTest

-- If you had accidentally left the last log with NORECOVERY then you could run this statement
-- at the very end, to bring the NewPubsTest database online

-- RESTORE DATABASE NewPubsTest WITH RECOVERY

-- Also, because this was a complete restore - you could have used STOPAT to return to an earlier 
-- point in time. Review the "Restore Log with Marked Transactions.sql" for information on how to 
-- use the STOPAT option.

-- Drop the Test Databases to free up disk space. You can always reproduce these scenarios by
-- re-running these scripts!

DROP DATABASE PubsTest, NewPubsTest

/*********************************************
******** ANSWER FOR QUESTION #2 ********
*********************************************/

-- 2) How can you recover up to the minute of the failure if you were to lose ONLY TestPubsFile2?

-- Re-run the "Restore File Backups.sql" script to recreate the same scenario as before.

-- Again, FIRST make sure you have the tail of the log:
BACKUP LOG [PubsTest] 
	TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
	WITH NAME = 'PubsTest Backup, FINAL Transaction Log (Tail up-to-minute)', NOINIT,
	NO_TRUNCATE
go

RESTORE DATABASE PubsTest
	FILE = 'PubsTestFile2'
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
WITH FILE = 3, -- this is the backup number by position
NORECOVERY
go

RESTORE DATABASE PubsTest
	FILE = 'PubsTestFile2'
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat'
WITH FILE = 9, -- this is the backup number by position
NORECOVERY
go

-- Since the pubs file is undamaged we only need the transaction logs AFTER the last 
-- differential. This time we'll start with the backup at position 10. However, if you 
-- want to test the transaction log backup at number 8 the error returned is very good.

-- RESTORE LOG PubsTest
-- FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
-- WITH FILE = 8, -- this is the backup number by position
-- NORECOVERY 
-- go

-- "The log in this backup set terminates at LSN 19000000171200001, which is too early to 
-- apply to the database. A more recent log backup that includes LSN 20000000161100002 
-- can be restored."

RESTORE LOG PubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH FILE = 10, -- this is the backup number by position
NORECOVERY 
go

RESTORE LOG PubsTest
FROM DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL\BACKUP\pubsbackup.dat' 
WITH FILE = 11, -- this is the backup number by position
RECOVERY 
go

DROP DATABASE PubsTest
go